%***********************************
%    EJERCICIO DEL MOTOR
% realimentación de estados
%***********************************
xdel(winsid())
clear;
clc;

% VALORES

Ra=7.94;
La=1.54e-3;
N=1/200;
Jm=26.6/(1000*100);
Bm=0;
Jc=48.5e-3;
Bc=660e-3;
Jeq=Jm+N^2*Jc;
Beq=Bm+N^2*Bc;
Kt=0.0393;
Kw=4.12e-3*60/2/%pi;
Ai=10;
N=1/200;
Ks=.159;
K1=0;
K2=0;
K3=0;
Ki=1;

// MATRICES DEL MODELO CONTINUO A LAZO ABIERTO

A=[-Ra/La,0,-Kw/La ;0,0,N;Kt/Jeq,0,-Beq/Jeq];
B=[Ai/La; 0; 0];
C=[1,0,0;0,0.1,0;0,0,N];
D=[0 ; 0; 0];

// MODELO DE ESTADO CONTINUO A LAZO ABIERTO

P1=syslin("c",A,B,C,D);
P1

// SIMULACIÓN

tiempo=[0:0.01:10]; // Vector de Tiempos
u=ones(1,length(tiempo));
Y=csim(u,tiempo,P1); // Simulación

// GRÁFICO VARIABLES DE ESTADO A LAZO ABIERTO 

subplot(3,1,1)
plot2d(tiempo,Y(1,:),color("red"));
xgrid(color("gray"))
title('Respuesta de las variables de estado')
ylabel("Corriente");
subplot(3,1,2)
plot2d(tiempo,Y(2,:),color("blue"));
xgrid(color("gray"))
ylabel("Ángulo");
subplot(3,1,3)
plot2d(tiempo,Y(3,:),color("scilabgreen4"));
xgrid(color("gray"))
ylabel("Velocidad");

// MODELO DE ESTADO DISCRETO
clc;
xdel(winsid())
Ts = 0.1;
disp('Modelo de estado discreto');
P1_dis =dscr(P1,Ts)
Ad=P1_dis.a;
Bd=P1_dis.b;
Cd=[0 1 0];
Dd=[0];
P2_dis=syslin("d",Ad,Bd,Cd,Dd)
U=cont_mat(Ad,Bd)
det(U)
rg=rank(U)
if rg==size(Ad)(1) then
    disp("El sistema es controlable")
end

// RESPUESTA DEL SISTEMA A LAZO CERADO CON Ks

G2=ss2tf(P2_dis);
Tlc1=G2/(1+G2*Ks)
tiempo=[0:Ts:40]; // Vector de Tiempos
u=ones(1,length(tiempo));
Ylc1=dsimul(tf2ss(Tlc1),u); // Simulación

// GRÁFICO DE LA SALIDA A LAZO CERRADO

plot2d(tiempo,Ylc1,color("red"));
xgrid(color("gray"))
title('Respuesta de la Posición Angular')


// CÁLCULO DEL MODELO CANÓNICO CONTROLABLE

clc;
den=coeff(G2.den)
num=coeff(G2.num)
Ac=[0,1,0;0,0,1;-den(1),-den(2),-den(3)];
Bc=[0;0;1];
Cc=[num(1),num(2),num(3)];
Dc=0;
str1=sprintf(' Modelo Canónico Controlable ');
disp(str1);
P3_dis=syslin("d",Ac,Bc,Cc,Dc)
Ucc=cont_mat(P3_dis)
det(Ucc)
str1=sprintf('Matriz de Transformación al MCC');
disp(str1);
Qcc_1=Ucc*U^-1

// MATRIZ A TRANSFORMADA

Acc=Qcc_1*Ad*Qcc_1^-1
Bcc=Qcc_1*Bd

// 
// PARÁMETROS PARA EL CÁLCULO DE LA REALIMENTACIÓN

// Se plantea un sistema con una transferencia a lazo cerrado
// con un Psi=0.6 y Wn=5 r/s.

Psi= 0.6;
Wn = 5;
p1 = -Psi*Wn+%i*Wn*sqrt(1-Psi^2)
p2 = -Psi*Wn-%i*Wn*sqrt(1-Psi^2)

// AGREGO LOS DOS AUTOVALORES RESTANTES CON PARTE REAL DE AL 
// MENOS 10 VECES MÁS G RANDE EN MÓDULO

p3 = 10*(-Psi*Wn)

// TRANSFORMO ESTOS AUTOVALORES AL PLANO Z

Pz=[exp(p1*Ts),exp(p2*Ts),exp(p3*Ts)]

// DISEÑO DE LA REALIMENTACIÓN DE ESTADOS

Pcr=poly(Pz,"z","roots");    // Polinomio deseado
Pcm=poly(Ad,"z");            // Polinomio de la planta
Pcd=Pcr-Pcm;                 // Diferencia
Ktcc=coeff(Pcd)              // Vector de realimentación del Mcc

Ktcc=ppol(Acc,Bcc,Pz) // Vector de realimentación del Mcc

Kt=Ktcc*Qcc_1

Kt=ppol(Ad,Bd,Pz) // Vector de realimentación 

// MODELO REALIMANTADO

Alc=Ad-Bd*Kt;
Clc=eye(3,3);
Dlc=zeros(3,1);
P4_dis=syslin("d",Alc,Bd,Clc,Dlc);
[ceros1,polos1,K1]=tf2zp(ss2tf(P4_dis))


// SIMULACIÓN

tiempo=[0:Ts:10]; // Vector de Tiempos
u=ones(1,length(tiempo));
Y=dsimul(P4_dis,u); // Simulación

// GRÁFICO VARIABLES DE ESTADO A LAZO ABIERTO 
xdel(winsid());
subplot(3,1,1)
plot2d(tiempo,Y(1,:),color("red"));
xgrid(color("gray"))
title('Respuesta de las variables de estado')
ylabel("Corriente");
subplot(3,1,2)
plot2d(tiempo,Y(2,:),color("blue"));
xgrid(color("gray"))
ylabel("Ángulo");
subplot(3,1,3)
plot2d(tiempo,Y(3,:),color("scilabgreen4"));
xgrid(color("gray"))
ylabel("Velocidad");

// AJUSTE DE LA GANANCIA Ki

Alc = [(Ad-Bd*Kt)]
Blc = [Bd]
Clc = [0 1 0]
Dlc = [0]

Klc=Clc*(eye(3)-Alc)^-1*Blc
A0=1/Klc
Kt0=Kt/A0

// CALCULO DEL VECTOR CON EL LAZO EXTERNO

Ks=0.159

Kts=Kt0-Ks/A0*Clc


